@@ -1,6 +1,6 @@
-#!/bin/bash
+#!/bin/sh
 
-# Options are passed on to deborphan
+# Most options are passed on to deborphan
 #
 # Orphaner is a neat frontend for deborphan displaying a list of
 # orphaned packages with dialog. Packages may be selected for removal
@@ -10,7 +10,8 @@
 #
 # Copyright (c) 2000 Goswin Brederlow <goswin.brederlow@student.uni-tuebingen.de>
 # Copyright (c) 2000, 2003, 2004, 2005, 2006 Peter Palfrader <peter@palfrader.org>
-# Copyright (c) 2003, 2004 Jrg Sommer <joerg@alea.gnuu.de>
+# Copyright (c) 2003, 2004, 2007, 2008 Jörg Sommer <joerg@alea.gnuu.de>
+# Copyright (c) 2008 Carsten Hey <c.hey@web.de>
 #
 # This program is dual licensed either unter the GNU GPL as published
 # by the Free Software Foundation; either version 2, or (at your option)
@@ -22,13 +23,13 @@
 # Debian systems. If you have problems obtaining the files please
 # write to the authors.
 #
-# $Id: orphaner.sh 593 2006-11-22 00:20:58Z weasel $
+# $Id: orphaner.sh 677 2008-05-27 18:38:25Z carsten-guest $
 
 set -e
 
 
 
-REVISION='$Rev: 593 $';
+REVISION='$Rev: 677 $';
 VERSION="(r${REVISION#\$Rev: }"
 VERSION="${VERSION% \$})"
 
@@ -36,7 +37,14 @@ OPTIONS=$@
 VALIDOPTIONS='^-([aDHns]|-libdevel|-guess-(.+)|-find-(.+)|-nice-mode|-all-packages|-priority(.+)|p(.+)|-show-section|-force-hold)[[:space:]]$'
 VALIDKEEPOPTIONS='^-([aDHns]|-libdevel|-guess-(.+)|-find-(.+)|-nice-mode|-all-packages|-priority(.+)|p(.+)|-show-section|-force-hold)[[:space:]]$'
 
-. gettext.sh
+if which gettext > /dev/null; then
+	. gettext.sh
+else
+	gettext() {
+		echo "$@"
+	}
+fi
+
 TEXTDOMAIN=deborphan
 export TEXTDOMAIN
 # xgettext:sh-format
@@ -62,7 +70,9 @@ NO_ORPHANS_FOUND=$(gettext 'No orphaned packages found.')
 # xgettext:no-sh-format
 DEBORPHAN_REMOVED=$(gettext 'deborphan got removed.  Exiting.')
 # xgettext:no-sh-format
-APT_GET_REMOVED=$(gettext 'deborphan got removed.  Exiting.')
+APT_GET_REMOVED=$(gettext 'apt got removed.  Exiting.')
+# xgettext:no-sh-format
+APT_GET_LOCKFAIL=$(gettext 'apt is not installed, broken dependencies found or could not open lock file, are you root?  Printing apt-get commandline and exiting:')
 # xgettext:no-sh-format
 REMOVING=$(gettext 'Removing %s')'\n'
 
@@ -72,6 +82,8 @@ DEBORPHAN_ERROR=$(gettext '"deborphan" returned with error.')
 APT_GET_ERROR=$(gettext '"apt-get" returned with exitcode %s.')'\n'
 # xgettext:sh-format
 DIALOG_ERROR=$(gettext '"dialog" returned with exitcode %s.')'\n'
+# xgettext:no-sh-format
+NUMBER_OF_PACKAGES_ERROR=$(gettext '"apt-get" tries to remove more packages than requested by orphaner.  Exiting.')'\n'
 
 # xgettext:no-sh-format
 SIMULATE_BUTTON=$(gettext 'Simulate')
@@ -100,7 +112,7 @@ if [ -n "${LINES:-}" -a -n "${COLUMNS:-}" ]; then
 	# Can't use LINES, because it colides with magic variable
 	# COLUMNS ditto
 	lines=$(($LINES - 7))
-	columns=$((COLUMNS - 10))
+	columns=$(($COLUMNS - 10))
 
 	# unset these magic variables to avoid unwished effects
 	unset LINES COLUMNS
@@ -121,7 +133,10 @@ else
 		fi
 	}
 
-	trap sigwinch_handle SIGWINCH
+	case "${BASH_VERSION:+bash}" in
+		bash) trap sigwinch_handle SIGWINCH;;
+		*) trap sigwinch_handle 20 28;;
+	esac
 fi
 
 if [ $lines -lt 12 -o $columns -lt 50 ]; then
@@ -132,7 +147,7 @@ fi
 LISTSIZE="$lines $columns $(($lines - 7))"
 BOXSIZE="$lines $columns"
 
-editkeepers() {
+editkeepers() { #{{{
 	for each in $OPTIONS; do
 		if [ "$SKIPONE" = "1" ]; then
 			SKIPONE=0;
@@ -183,9 +198,9 @@ editkeepers() {
 		esac
 		rm $PACKAGES
 	fi
-}
+} #}}}
 
-keeping_list() {
+keeping_list() { #{{{
 	{
 		{ deborphan -a $@ || echo "ERROR"; } \
 			| while read SECTION PACKAGE; do
@@ -196,17 +211,17 @@ keeping_list() {
 			echo $PACKAGE "." on
 		done
 	} | sort
-}
+} #}}}
 
-deborphan_list() {
+deborphan_list() { #{{{
 	{ deborphan -s $@ || echo "ERROR"; } \
 		| while read SECTION PACKAGE; do
 		echo $PACKAGE $SECTION off
 	done
-}
+} #}}}
 
-doorphans() {
-	# Check options
+doorphans() { #{{{
+	# Check options {{{
 	skipone=0
 	for each in $OPTIONS; do
 		if [ "$skipone" = "1" ]; then
@@ -225,10 +240,10 @@ doorphans() {
 					;;
 			esac
 		fi
-	done
+	done #}}}
 
 	TMPFILE=`tempfile`
-	trap "if [ -e $TMPFILE ]; then rm $TMPFILE; fi" EXIT
+	trap "rm -f $TMPFILE" EXIT INT
 
 	EXCLUDE=
 	ORPHANED=
@@ -244,20 +259,17 @@ doorphans() {
 			exit 1
 		fi
 
-		if [ -z "$ORPHANED$EXCLUDE" ]; then
+		if [ -z "$ORPHANED$EXCLUDE" ]; then #{{{# nothing to do
 			dialog \
 				--backtitle "Orphaner $VERSION" \
 				--title "Orphaner $VERSION" \
 				--msgbox "$NO_ORPHANS_FOUND" \
 				$BOXSIZE
-			break
-		elif [ -z "$OLD_ORPHANED" ]; then
-			# it's the first loop cycle
+			break #}}}
+		elif [ -z "$OLD_ORPHANED" ]; then #{{{# it's the first loop cycle
 			SPLIT_NEW=
-			SPLIT_OLD="$ORPHANED"
-		elif [ -z "$ORPHANED" ]; then
-			# maybe we have excluded all packages and no new packages
-			# were orphaned
+			SPLIT_OLD="$ORPHANED" #}}}
+		elif [ -z "$ORPHANED" ]; then #{{{# maybe we have excluded all packages and no new packages were orphaned
 			ORPHANED="$OLD_ORPHANED"
 			SPLIT_NEW=
 			SPLIT_OLD=
@@ -267,8 +279,8 @@ doorphans() {
 $OLD_ORPHANED
 __OORPH_EOT
 
-			SPLIT_OLD="${SPLIT_OLD#$NL}" # trim leading newline character
-		else
+			SPLIT_OLD="${SPLIT_OLD#$NL}" # trim leading newline character }}}
+		else #{{{# normal loop cycle
 			# Idea: you have two sorted lists: the list of the
 			# orphaned packages in the last cycle and the list of
 			# orphaned packages in this cycle. Now you compare element
@@ -289,7 +301,7 @@ __OORPH_EOT
 				# scratch
 				ORPHANED=
 			fi
-			while true; do
+			while true; do #{{{
 				if [ "$LINE" ">" "$OLD_LINE" ]; then
 					# The package from the old orphaned list was removed
 					if [ -n "$EXCLUDE" ]; then
@@ -330,7 +342,7 @@ __OORPH_EOT
 						break
 					fi
 				fi
-			done
+			done #}}}
 			exec 4<&-
 
 			# The list of old orphaned packages reached the end. So
@@ -353,8 +365,9 @@ __OORPH_EOT
 			ORPHANED="${ORPHANED#$NL}"
 			SPLIT_OLD="${SPLIT_OLD#$NL}"
 			SPLIT_NEW="${SPLIT_NEW#$NL}"
-		fi
+		fi #}}}
 
+		# Display dialog box and handle buttons {{{
 		while true; do
 			ERROR=0
 			dialog --backtitle "Orphaner $VERSION" \
@@ -370,15 +383,32 @@ __OORPH_EOT
 			unset DEFAULT_PKG EXCLUDE
 
 			case $ERROR in
-				0) # OK-Button
+				0) # OK-Button {{{
 					if [ ! -s $TMPFILE ]; then
 						# nothing's selected
 						break 2
 					fi
 					clear
-					printf $REMOVING "$(cat $TMPFILE)"
-					apt-get $PURGE --show-upgraded --assume-yes remove \
-						$(cat $TMPFILE) || ERROR=$?
+					# tr , ' ' is used for compatibility with the svn branch deborphan-2.0
+					PACKAGES_TO_REMOVE="$(printf '%s ' $(grep -v '^----$' $TMPFILE | tr , ' '))"
+					PACKAGES_TO_REMOVE="${PACKAGES_TO_REMOVE% }"
+#					printf "$REMOVING" "$PACKAGES_TO_REMOVE"
+					APT_GET_CMDLN="apt-get $PURGE --show-upgraded --assume-yes remove $PACKAGES_TO_REMOVE"
+					if apt-get check -q -q; then
+						LC_ALL=C $APT_GET_CMDLN --simulate \
+						| grep -q "^0 upgraded, 0 newly installed, `echo "$PACKAGES_TO_REMOVE" | wc -w` to remove and"
+						if [ $? -ne 0 ]; then
+							LC_ALL=C $APT_GET_CMDLN --simulate >&2
+							printf '%s\n' "$NUMBER_OF_PACKAGES_ERROR" >&2
+							exit 1
+						fi
+						$APT_GET_CMDLN || ERROR=$?
+					else
+						printf '%s\n' "$APT_GET_LOCKFAIL" >&2
+						printf '%s\n' "$APT_GET_CMDLN" >&2
+						exit 1
+					fi
+					unset APT_GET_CMDLN PACKAGES_TO_REMOVE
 					if [ $ERROR -ne 0 ]; then
 						printf "$APT_GET_ERROR" $ERROR >&2
 						exit 1
@@ -393,13 +423,13 @@ __OORPH_EOT
 					fi
 					echo
 					echo "$PRESS_ENTER_TO_CONTINUE"
-					read -s UNUSED_VARIABLE_NAME
+					read UNUSED_VARIABLE_NAME
 					break
-					;;
-				1) # Cancel-Button
+					;; #}}}
+				1) # Cancel-Button #{{{
 					break 2
-					;;
-				2) # Help-Button
+					;; #}}}
+				2) # Help-Button #{{{
 					SEL_LIST=
 					while read pkg; do
 						case "$pkg" in
@@ -468,25 +498,25 @@ __EOT
 					dialog --backtitle "Orphaner $VERSION" \
 						--title "Orphaner $VERSION" \
 						--textbox $TMPFILE $BOXSIZE
-					;;
-				3) # Simulate-Button
+					;; #}}}
+				3) # Simulate-Button #{{{
 					EXCLUDE=$(while read pkg; do printf $pkg,; done < $TMPFILE)
 					EXCLUDE=${EXCLUDE%,}
 					break
-					;;
-				*)
+					;; #}}}
+				*) #{{{
 					printf "$DIALOG_ERROR" $ERROR >&2
 					cat $TMPFILE
-					exit 1
+					exit 1 #}}}
 			esac
-		done
-	done
-}
+		done #}}}
+	done #}}}
+} #}}}
 
 # purge set?
 case " $OPTIONS " in
 	*" --purge "*)
-		OPTIONS="${OPTIONS%--purge*}${OPTIONS#*--purge}"
+		OPTIONS="${OPTIONS%%--purge*}${OPTIONS#*--purge}"
 		PURGE=--purge
 		;;
 esac
